
/*
 * Biblioteca para ler informacoes do /proc
 * Marcelo Veiga Neves
 * veiga@inf.ufsm.br
 * Tue Mar 2 00:33:5 2004
 */

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>
#include <dirent.h>
#include <errno.h>
#include <utmp.h>
#include "proc.h"

double _interval = DEFAULT_TIME;


double _proc_now(void)
{
    struct timeval rt;
    gettimeofday(&rt, NULL);
    return (rt.tv_sec * 1000000.0 + rt.tv_usec);
}

double _proc_randon_interval(int min, int max)
{
    if (min == max) {
	return min;
    }
    srand((int) ((long long) _proc_now() % 100000));
    return (min + (((double) (max - min)) * rand() / (RAND_MAX + 1.0)));
}

/**
 * Set interval between readings (microseconds). 
 * If a interval is not specified then DEFAULT_TIME is assumed.
 *
 * @brief Set Collect Interval
 * @param interval interval (microseconds)
 */
void proc_set_interval(double interval)
{
    if (interval >= 0)
	_interval = interval;
}


/* le um inteiro do inicio do arquivo, em caso de erro retorna -1
 */
int _proc_read_integer(char *filename)
{
    FILE *fp;
    int p;
    fp = fopen(filename, "r");
    if (!fp)
	return -1;
    fscanf(fp, "%d", &p);
    fclose(fp);
    return p;
}

/* le um string do inicio do arquivo, retorna o numero de caracteres lidos ou
 * 0 em caso de erro
 */
int _proc_read_string(char *filename, char *s)
{
    FILE *fp;
    fp = fopen(filename, "r");
    if (!fp)
	return 0;
    fscanf(fp, "%s", s);
    fclose(fp);
    return strlen(s);
}

/* le um linha do inicio do arquivo e coloca no string s, retorna o numero de
 * caracteres lidos ou 0 em caso de erro
 */
int _proc_read_line(char *filename, char *s)
{
    FILE *fp;
    char c;
    int i = 0;

    fp = fopen(filename, "r");
    if (!fp)
	return 0;

    do {
	c = getc(fp);
	if (c == '\0')
	    c = ' ';
	*(s + i) = c;
	i++;
    } while (!feof(fp) && c != '\n');
    *(s + i - 1) = '\0';
    fclose(fp);

    return strlen(s);
}

/**
 * The number of seconds the kernel will postpone rebooting the system when a 
 * kernel panic is experienced. By default, the value is set to 0, which 
 * disables automatic rebooting after a panic. 
 *
 * @brief Timeout after a Kernel Panic
 * @return On success, returns Timeout after a Kernel Panic.
 * @return On error, -1 is returned.
 */
int proc_panic(void)
{
    errno = 0;
    return _proc_read_integer(PANIC);
}

/**
 * The maximum number of threads to be used by the kernel, with a default value
 *  of 4095. 
 *
 * @brief Maximum number of threads to be used by the kernel
 * @return On success, returns the thread max value.
 * @return On error, -1 is returned.
 */
int proc_threads_max(void)
{
    errno = 0;
    return _proc_read_integer(THREADMAX);
}

/**
 * The system's domain name, such as domain.com.
 *
 * @brief  The system's domain name
 * @param s buffer to put the domainname
 * @return On success, returns the domainname length. 
 * @return On error, -1 is returned.
 * @see proc_hostname proc_ip
 */
int proc_domainname(char *s)
{
    errno = 0;
    return _proc_read_string(DOMAINNAME, s);
}


/**
 * The system's host name, such as host.domain.com. 
 *
 * @brief The system's host name
 * @param s buffer to put the hostname
 * @return On success, returns the hostname length.
 * @return On error, -1 is returned.
 * @see proc_domainname proc_ip
 */
int proc_hostname(char *s)
{
    errno = 0;
    return _proc_read_string(HOSTNAME, s);
}


/**
 * Operating System Type
 *
 * @brief Operating System Type
 * @param s buffer para colocar o OS type
 * @return Return the OS type length.
 * @return On error, -1 is returned.
 * @see proc_osname proc_osrelease
 */
int proc_ostype(char *s)
{
    errno = 0;
    return _proc_read_string(OSTYPE, s);
}


/**
 * Kernel Command Line Arguments.
 *
 * @brief Kernel Command Line
 * @param s buffer to put the cmdline
 * @return On success, returns the command line length.
 * @return On error, -1 is returned.
 * @see proc_osname proc_osrelease
 */
int proc_cmdline(char *s)
{
    errno = 0;
    return _proc_read_line(CMDLINE, s);
}


/**
 * Operating System Release.
 *
 * @brief Operating System Release
 * @param s buffer to put the OS release
 * @return On success, returns the OS release length.
 * @return On error, -1 is returned.
 * @see proc_osname proc_ostype
 */
int proc_osrelease(char *s)
{
    errno = 0;
    return _proc_read_string(OSRELEASE, s);
}


uptime_t uptime = { 0, -1, -1 };

/* le o uptime (tempo que passou desde a inicializacao da maquina) e o 
 * uptime_idle (tempo que a maquina ficou ociosa dede a inicializacao) 
 * A funcao retorna 0 em caso de erro ou 1 em caso de sucesso.
 */
int _proc_read_uptime(uptime_t * up)
{
    FILE *fp;

    fp = fopen(UPTIME, "r");
    if (!fp)
	return 0;

    if (fscanf(fp, "%lf %lf", &up->uptime, &up->uptime_idle) != 2) {
	fclose(fp);
	return 0;
    }
    fclose(fp);
    up->last = _proc_now();
    return 1;
}


/**
 * The total number of seconds the system has been up.
 *
 * @brief Uptime
 * @return On success, returns the system uptime.
 * @return On error, -1 is returned.
 * @see proc_uptime_idle
 */
double proc_uptime(void)
{
    errno = 0;
    if ((uptime.last == 0) || (_proc_now() > uptime.last + _interval)) {
	if (!_proc_read_uptime(&uptime))
	    return -1;
    }
    return uptime.uptime;
}


/**
 * How much of that time, since last restart, the machine has spent idle.
 *
 * @brief Idle uptime
 * @return On success, returns the machine idle uptime.
 * @return On error, -1 is returned.
 * @see proc_uptime
 */
double proc_uptime_idle(void)
{
    errno = 0;
    if ((uptime.last == 0) || (_proc_now() > uptime.last + _interval)) {
	if (!_proc_read_uptime(&uptime))
	    return -1;
    }
    return uptime.uptime_idle;
}


/**
 *  The maximum number of file handles that the kernel will allocate.
 *
 * @brief Maximum number of file handles
 * @return On success, returns file max.
 * @return On error, -1 is returned.
 * @see proc_fd_system proc_fd_disp proc_fd_limit
 */
int proc_fd_max(void)
{
    errno = 0;
    return _proc_read_integer(FILE_MAX);
}


fd_t fd = { 0, -1, -1, -1 };

/* le informacoes sobre descritores de arquivos (fd do sistema,
 * fd em uso e limite) do arquivo /proc/sys/fs/file-nr, a funcao
 * retorna 1 em caso de sucesso e 0 em caso de erro.
 */
int _proc_read_fd(fd_t * fd)
{
    FILE *fp;
    fp = fopen(FILE_NR, "r");
    if (!fp)
	return 0;
    if (fscanf(fp, "%d %d %d", &fd->system, &fd->disp, &fd->limit) != 3) {
	fclose(fp);
	return 0;
    }
    fclose(fp);
    fd->last = _proc_now();
    return 1;
}


/**
 * The number of allocated file handles by the system.
 *
 * @brief The number of allocated file handles
 * @return On success, returns the number of allocated file handles.
 * @return On error, -1 is returned.
 * @see proc_fd_max proc_fd_disp proc_fd_limit
 */
int proc_fd_system(void)
{
    errno = 0;
    if ((fd.last == 0) || (_proc_now() > fd.last + _interval)) {
	if (!_proc_read_fd(&fd))
	    return -1;
    }
    return fd.system;
}


/**
 * The number of Available file handles.
 *
 * @brief The number of available file handles
 * @return On success, returns the fd max.
 * @return On error, -1 is returned.
 * @see proc_fd_max proc_fd_system proc_fd_limit
 */
int proc_fd_disp(void)
{
    errno = 0;
    if ((fd.last == 0) || (_proc_now() > fd.last + _interval)) {
	if (!_proc_read_fd(&fd))
	    return -1;
    }
    return fd.disp;
}


/**
 * The maximum number of file handles.
 *
 * @brief The maximum number of file handles
 * @return On success, returns fd limit.
 * @return On error, -1 is returned.
 * @see proc_fd_max proc_fd_system proc_fd_disp
 */
int proc_fd_limit(void)
{
    errno = 0;
    if ((fd.last == 0) || (_proc_now() > fd.last + _interval)) {
	if (!_proc_read_fd(&fd))
	    return -1;
    }
    return fd.limit;
}

inode_t inode = { 0, -1, -1 };

/* le informacaoes sobre inodes (inodes so estruturas de dados
 * que contm informaes sobre os arquivos em um sistema como, por
 * exemplo, quem  o dono dos arquivos, a quais grupos pertencem, 
 * data da ltima modificao, quem pode ler ou executar estes 
 * arquivos e assim por diante) do arquivo /proc/sys/fs/inode-nr,
 * a funcao preenche uma struct inode_t. (obs: na scruct alloc 
 * representa o nr_inodes e free representa o nr_inode_free)
 */
int _proc_read_inode(inode_t * inode)
{
    FILE *fp;
    fp = fopen(INODE_NR, "r");
    if (!fp)
	return 0;
    if (fscanf(fp, "%d %d", &inode->alloc, &inode->free) != 2) {
	fclose(fp);
	return 0;
    }
    fclose(fp);
    inode->last = _proc_now();
    return 1;
}

/**
 * The number of inodes the system has allocated.
 *
 * @brief The number of inodes the system has allocated
 * @return On success, returns the number of inode allocated.
 * @return On error, -1 is returned.
 * @see proc_inode_inuse proc_inode_free
 */
int proc_inode_alloc(void)
{
    errno = 0;
    if ((inode.last == 0) || (_proc_now() > inode.last + _interval)) {
	if (!_proc_read_inode(&inode))
	    return -1;
    }
    return inode.alloc;
}

/**
 * The number of free inodes.
 *
 * @brief The number of free inodes
 * @return On success, returns the number of free inodes.
 * @return On error, -1 is returned.
 * @see proc_inode_inuse proc_inode_alloc
 */
int proc_inode_free(void)
{
    errno = 0;
    if ((inode.last == 0) || (_proc_now() > inode.last + _interval)) {
	if (!_proc_read_inode(&inode))
	    return -1;
    }
    return inode.free;
}

/**
 * The number of inodes in use.
 *
 * @brief The number of inodes in use
 * @return On success, returns the number of inode in use.
 * @return On error, -1 is returned.
 * @see proc_inode_alloc proc_inode_free
 */
int proc_inode_inuse(void)
{
    errno = 0;
    if ((inode.last == 0) || (_proc_now() > inode.last + _interval)) {
	if (!_proc_read_inode(&inode))
	    return -1;
    }
    return (inode.alloc - inode.free);
}

load_t load = { 0, -1, -1, -1, -1, -1, -1 };

/* le a "carga mdia" do sistema, do arquivo /proc/loadavg, contendo
 * trs indicadores de quanto trabalho o sistema est executando no momento.
 * a funcao preenche uma struct load_t contendo a mdia do
 * nmero de servios na fila de execuo h mais de 1, 5 e 15 minutos 
 * (one, five, fifteen), em caso de erro a funcao retorna 0 ou 1 para sucesso.
 */
int _proc_read_load(load_t * load)
{
    FILE *fp;
    fp = fopen(LOADAVG, "r");
    if (!fp)
	return 0;
    if (fscanf(fp, "%f %f %f %d/%d %d", &load->one, &load->five,
	       &load->fifteen, &load->running, &load->total,
	       &load->last_id) != 6) {
	fclose(fp);
	return 0;
    }
    fclose(fp);
    load->last = _proc_now();
    return 1;
}

/**
 * The value of average system load for the last 1 minute. This value is 
 * updated each timer interrupt.
 *
 * @brief One minute load average
 * @return On success, returns the one minute load average.
 * @return On error, -1 is returned.
 * @see proc_load_five proc_load_fifteen
 * @see proc_process_running proc_process_total proc_process_last
 */
float proc_load_one(void)
{
    errno = 0;
    if ((load.last == 0) || (_proc_now() > load.last + _interval)) {
	if (!_proc_read_load(&load))
	    return -1;
    }
    return load.one;
}

/**
 * The value of average system load for the last 5 minute. This value is 
 * updated each timer interrupt.
 *
 * @brief Five minute load average
 * @return On success, returns the five minute load average.
 * @return On error, -1 is returned.
 * @see proc_load_one proc_load_fifteen 
 * @see proc_process_running proc_process_total proc_process_last
 */
float proc_load_five(void)
{
    errno = 0;
    if ((load.last == 0) || (_proc_now() > load.last + _interval)) {
	if (!_proc_read_load(&load))
	    return -1;
    }
    return load.five;
}

/**
 * The value of average system load for the last 15 minute. This value is 
 * updated each timer interrupt.
 *
 * @brief Fifteen minute load average
 * @return On success, returns the fifteen minute load average.
 * @return On error, -1 is returned.
 * @see proc_load_one proc_load_five
 * @see proc_process_running proc_process_total proc_process_last
 */
float proc_load_fifteen(void)
{
    errno = 0;
    if ((load.last == 0) || (_proc_now() > load.last + _interval)) {
	if (!_proc_read_load(&load))
	    return -1;
    }
    return load.fifteen;
}

/**
 * The number of currently running processes. This value is updated each timer interrupt.
 *
 * @brief The number of running process
 * @return On success, returns the number of running process.
 * @return On error, -1 is returned.
 * @see proc_load_one proc_load_five proc_load_fifteen
 * @see proc_process_total proc_process_last
 */
int proc_process_running(void)
{
    errno = 0;
    if ((load.last == 0) || (_proc_now() > load.last + _interval)) {
	if (!_proc_read_load(&load))
	    return -1;
    }
    return load.running;
}

/**
 * The total number of processes. This value is updated each timer interrupt.
 *
 * @brief The total number of processes
 * @return On success, returns the total number of process.
 * @return On error, -1 is returned.
 * @see proc_load_one proc_load_five proc_load_fifteen
 * @see proc_process_running proc_process_last
 */
int proc_process_total(void)
{
    errno = 0;
    if ((load.last == 0) || (_proc_now() > load.last + _interval)) {
	if (!_proc_read_load(&load))
	    return -1;
    }
    return load.total;
}

/**
 * The PID of the last active process. This value is updated each timer interrupt.
 *
 * @brief The PID of the last active process
 * @return On success, returns the PID of the last active process.
 * @return On error, -1 is returned.
 * @see proc_load_one proc_load_five proc_load_fifteen
 * @see proc_process_running proc_process_total
 */
int proc_process_last(void)
{
    errno = 0;
    if ((load.last == 0) || (_proc_now() > load.last + _interval)) {
	if (!_proc_read_load(&load))
	    return -1;
    }
    return load.last_id;
}

meminfo_t meminfo =
    { 0, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-1, -1 };

/* le informacoes sobre a memoria (total, usada, livre, compartilhada,
 * buffers, cached e porcentagem de uso) do arquivo /proc/meminfo, 
 * retorna 1 em caso de sucesso ou 0 em caso de erro
 */
int _proc_read_meminfo(meminfo_t * meminfo)
{
    FILE *fp;
    char buff[30];
    int value;
    fp = fopen(MEMINFO, "r");
    if (!fp)
	return 0;

    while ((fscanf(fp, "%s", buff) == 1) && strcmp(buff, "MemTotal:"));

    fscanf(fp, "%d %*s", &meminfo->mem_total);

    //printf("mem_total = %d\n", meminfo->mem_total);

    fscanf(fp, "%s %d %*s", buff, &value);
    do {
	if (strcmp(buff, "MemFree:") == 0) {
	    meminfo->mem_free = value;
	    //printf("mem_free = %d\n", meminfo->mem_free);
	} else if (strcmp(buff, "MemShared:") == 0) {
	    meminfo->mem_shared = value;
	    //printf("mem_shared = %d\n", meminfo->mem_shared);
	} else if (strcmp(buff, "Buffers:") == 0) {
	    meminfo->buffers = value;
	    //printf("buffers = %d\n", meminfo->buffers);
	} else if (strcmp(buff, "Cached:") == 0) {
	    meminfo->cached = value;
	    //printf("cached = %d\n", meminfo->cached);
	} else if (strcmp(buff, "SwapCached:") == 0) {
	    meminfo->swap_cached = value;
	    //printf("swap_cached = %d\n", meminfo->swap_cached);
	} else if (strcmp(buff, "Active:") == 0) {
	    meminfo->active = value;
	    //printf("active = %d\n", meminfo->active);
	} else if (strcmp(buff, "Inactive:") == 0) {
	    meminfo->inactive = value;
	    //printf("inactive = %d\n", meminfo->inactive);
	} else if (strcmp(buff, "HighTotal:") == 0) {
	    meminfo->high_total = value;
	    //printf("high_total = %d\n", meminfo->high_total);
	} else if (strcmp(buff, "HighFree:") == 0) {
	    meminfo->high_free = value;
	    //printf("high_free = %d\n", meminfo->high_free);
	} else if (strcmp(buff, "LowTotal:") == 0) {
	    meminfo->low_total = value;
	    //printf("low_total = %d\n", meminfo->low_total);
	} else if (strcmp(buff, "LowFree:") == 0) {
	    meminfo->low_free = value;
	    //printf("low_free = %d\n", meminfo->low_free);
	} else if (strcmp(buff, "SwapTotal:") == 0) {
	    meminfo->swap_total = value;
	    //printf("swap_total = %d\n", meminfo->swap_total);
	} else if (strcmp(buff, "SwapFree:") == 0) {
	    meminfo->swap_free = value;
	    //printf("swap_free = %d\n", meminfo->swap_free);
	}

    } while (fscanf(fp, "%s %d %*s", buff, &value) == 2);

    /*
       fscanf(fp,"%d %d %d %d", &mem->total, &mem->used, &mem->free, &mem->shared);
       fscanf(fp,"%d %d", &mem->buffers, &mem->cached);
     */
    meminfo->mem_used = meminfo->active;
    meminfo->swap_used = meminfo->swap_total - meminfo->swap_free;
    meminfo->mem_pused =
	(meminfo->mem_used * 100 / (double) meminfo->mem_total);
    meminfo->swap_pused =
	(meminfo->swap_used * 100 / (double) meminfo->swap_total);

    meminfo->last = _proc_now();

    fclose(fp);
    return 1;
}

/**
 * Total amount of physical RAM, in kilobytes.
 *
 * @brief The number of total bytes of main memory
 * @return On success, returns total memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_used proc_mem_buffers
 * @see proc_mem_cached proc_mem_active proc_mem_inactive proc_mem_high_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused
 */
int proc_mem_total(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.mem_total;
}

/**
 * The amount of physical RAM, in kilobytes, left unused by the system. 
 *
 * @brief The number of free bytes of main memory
 * @return On success, returns free memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_total proc_mem_shared proc_mem_used proc_mem_buffers
 * @see proc_mem_cached proc_mem_active proc_mem_inactive proc_mem_high_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused
 */
int proc_mem_free(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.mem_free;
}

/**
 * The amount of physical RAM, in kilobytes, used by the system. 
 *
 * @brief The number of used bytes of main memory
 * @return On success, returns used memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_total proc_mem_buffers
 * @see proc_mem_cached proc_mem_active proc_mem_inactive proc_mem_high_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused proc_mem_pused_pid
 */
int proc_mem_used(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.mem_used;
}

/**
 * The number of shared bytes of main memory.
 *
 * @brief The number of shared bytes of main memory
 * @return On success, returns shared memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_total proc_mem_used proc_mem_buffers
 * @see proc_mem_cached proc_mem_active proc_mem_inactive proc_mem_high_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused proc_mem_pused_pid
 */
int proc_mem_shared(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.mem_shared;
}

/**
 * The amount of physical RAM, in kilobytes, used for file buffers.
 *
 * @brief The number of buffered bytes of main memory
 * @return On success, returns buffered memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_used proc_mem_total
 * @see proc_mem_cached proc_mem_active proc_mem_inactive proc_mem_high_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused, proc_mem_pused_pid
 */
int proc_mem_buffers(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.buffers;
}

/**
 * The amount of physical RAM, in kilobytes, used as cache memory.
 *
 * @brief The number of cached bytes of main memory
 * @return On success, returns cached memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_used proc_mem_buffers
 * @see proc_mem_total proc_mem_active proc_mem_inactive proc_mem_high_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused proc_mem_pused_pid
 */
int proc_mem_cached(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.cached;
}

/**
 * The total amount of buffer or page cache memory, in kilobytes, that is in active use.
 *
 * @brief The amount of memory in active use. 
 * @return On success, returns active memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_used proc_mem_buffers
 * @see proc_mem_cached proc_mem_total proc_mem_inactive proc_mem_high_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused proc_mem_pused_pid
 */
int proc_mem_active(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.active;
}

/**
 * The total amount of buffer or cache pages, in kilobytes, that might be freeable or available.
 *
 * @brief The amount of inactive memory. 
 * @return On success, returns inactive memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_used proc_mem_buffers
 * @see proc_mem_cached proc_mem_active proc_mem_total proc_mem_high_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused
 */
int proc_mem_inactive(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.inactive;
}

/**
 * The total amount of memory that is not directly mapped into kernel space.
 *
 * @return On success, returns high total memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_used proc_mem_buffers
 * @see proc_mem_cached proc_mem_active proc_mem_inactive proc_mem_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused
 */
int proc_mem_high_total(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.high_total;
}

/**
 * The amount of free memory that is not directly mapped into kernel space.
 *
 * @return On success, returns high free memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_used proc_mem_buffers
 * @see proc_mem_cached proc_mem_active proc_mem_inactive proc_mem_high_total
 * @see proc_mem_total proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused
 */
int proc_mem_high_free(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.high_free;
}

/**
 * The total amount of memory that is directly mapped into kernel space.
 *
 * @return On success, returns low total memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_used proc_mem_buffers
 * @see proc_mem_cached proc_mem_active proc_mem_inactive proc_mem_total
 * @see proc_mem_high_free proc_mem_total proc_mem_low_free
 * @see proc_mem_pused
 */
int proc_mem_low_total(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.low_total;
}

/**
 * The amount of free memory that is directly mapped into kernel space.
 *
 * @return On success, returns low free memory (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_used proc_mem_buffers
 * @see proc_mem_cached proc_mem_active proc_mem_inactive proc_mem_high_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_total
 * @see proc_mem_pused
 */
int proc_mem_low_free(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.low_free;
}

/**
 * The total amount of swap available, in kilobytes.
 *
 * @brief The total amount of swap available
 * @return On success, returns the total amount of swap (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_swap_free proc_swap_used proc_swap_cached proc_swap_pused 
 * @see proc_mem_total proc_mem_free
 */
int proc_swap_total(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.swap_total;
}

/**
 * The total amount of swap free, in kilobytes.
 *
 * @brief The total amount of swap free
 * @return On success, returns the total amount of swap free (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_swap_total proc_swap_used proc_swap_cached proc_swap_pused 
 * @see proc_mem_total proc_mem_free
 */
int proc_swap_free(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.swap_free;
}

/**
 * The total amount of used swap, in kilobytes.
 *
 * @brief The total amount of used swap
 * @return On success, returns the total amount of used swap (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_swap_total proc_swap_free proc_swap_cached proc_swap_pused 
 * @see proc_mem_total proc_mem_free
 */
int proc_swap_used(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.swap_used;
}

/**
 * The total amount of cached swap, in kilobytes.
 *
 * @brief The total amount of cached swap
 * @return On success, returns the total amount of cached swap (in Kbytes).
 * @return On error, -1 is returned.
 * @see proc_swap_total proc_swap_used proc_swap_free proc_swap_pused 
 * @see proc_mem_total proc_mem_free
 */
int proc_swap_cached(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.swap_cached;
}

/**
 * Percentage of used memory.
 *
 * @brief Percentage of used memory
 * @return On success, returns percentage of used memory.
 * @return On error, -1 is returned.
 * @see proc_mem_free proc_mem_shared proc_mem_used proc_mem_buffers
 * @see proc_mem_cached proc_mem_active proc_mem_inactive proc_mem_high_total
 * @see proc_mem_high_free proc_mem_low_total proc_mem_low_free
 * @see proc_mem_pused
 */
double proc_mem_pused(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.mem_pused;
}

/**
 * Percentage of used swap.
 *
 * @brief Percentage of used swap
 * @return On success, returns percentage of used memory.
 * @return On error, -1 is returned.
 * @see proc_swap_total proc_swap_used proc_swap_free proc_swap_pused 
 * @see proc_mem_total proc_mem_free
 */
double proc_swap_pused(void)
{
    errno = 0;
    if ((meminfo.last == 0) || (_proc_now() > meminfo.last + _interval)) {
	if (!_proc_read_meminfo(&meminfo))
	    return -1;
    }
    return meminfo.swap_pused;
}

/* retira os parenteses,
 * usada pela funcao proc_getpid e _proc_read_stat_pid()
 */
void _proc_clean_name(char *process)
{
    int len, i;

    len = strlen(process);
    for (i = 1; i < len - 1; i++) {
	process[i - 1] = process[i];
    }
    process[len - 2] = '\0';
}

/**
 * Get PID of a process (with name equal "exe"). If there are some PIDs for 
 * one executable name, the function returns the PID of the last process created.
 *
 * @brief Get PID by name
 * @author Added by Tiago Scheid <scheid(at)inf.ufsm.br>
 * @return On success, returns the PID of process exe.
 * @return On error, -1 is returned.
 * @see proc_pid_exists proc_exe_name_pid proc_username_pid proc_exe_pid 
 * @see proc_cmdline_pid
 */
int proc_getpid(char *exe)
{
    /* retorna o pid do processo de nome exe se tiver mais de um processo com esse
    * nome, a funcao vai retornar o de maior PID, a funcao retorna -1 em caso de erro.
    * Funcao adicionado por Tiago Scheid <scheid@inf.ufsm.br>
    */
    
    DIR *proc;			/* /proc/ */
    FILE *file_stat;		/* /proc/{pid}/stat */
    char buffer_pid[25];
    char name[30];
    int pid = -1;
    int tmp;
    struct dirent *dir;

    if ((proc = opendir("/proc")) == NULL) {
	return -1;
    }

    /* le tudo o que ha no diretorio, seleciona o que eh pid */
    while ((dir = readdir(proc)) != NULL) {
	/* so entra nos diretorio que correspondem a um pid */
	if (isdigit((*dir).d_name[0])) {
	    sprintf(buffer_pid, "/proc/%s/stat", (*dir).d_name);
	    /* buffer contem "/proc/{pid}/stat" */
	    file_stat = fopen(buffer_pid, "r");
	    fscanf(file_stat, "%*s %s", name);
	    _proc_clean_name(name);
	    /* compara com o nome do executavel */
	    if (!strcmp(exe, name)) {
		tmp = atoi((*dir).d_name);
		/* pega o ultimo pid */
		if (tmp > pid)
		    pid = tmp;
	    }
	    fclose(file_stat);
	}

    }
    closedir(proc);
    return pid;
}

time_t proc_time(void)
{
    errno = 0;
    return time(0);
}

stat_t stat = { 0, -1, -1, -1, -1, -1, -1, -1, -1 };

/* le o conteudo do arquivo /proc/stat:
 * intr -  nmero de interrupes recebidas a partir da inicializao do sistema
 * page -  nmero de pginas que entraram no sistema e o nmero de pginas que 
 * sairam (do disco). 
 * ctxt -  nmero de mudanas de contexto que o sistema realizou (desde a 
 * inicializacao?), 
 * Em caso de erro de leitura do arquivo retorna 0 e 1 para sucesso.
 * Essa funcao tambem analisa o arquivo /proc/stat para contar o numero de CPUs
 */
int _proc_read_stat(stat_t * stat)
{
    FILE *fp;
    char buff[30];

    fp = fopen(STAT, "r");
    if (!fp)
	return 0;
    stat->cpu_num = -1;
    fscanf(fp, "%s", buff);
    do {
	if (strcmp(buff, "page") == 0) {
	    fscanf(fp, "%d %d", &stat->page_in, &stat->page_out);
	    //printf("page_in = %d\npage_out = %d\n", stat->page_in, stat->page_out);
	} else if (strcmp(buff, "swap") == 0) {
	    fscanf(fp, "%d %d", &stat->swap_in, &stat->swap_out);
	    //printf("swap_in = %d\nswap_out = %d\n", stat->swap_in, stat->swap_out);
	} else if (strcmp(buff, "ctxt") == 0) {
	    fscanf(fp, "%d", &stat->ctxt);
	    //printf("ctxt = %d\n", stat->ctxt);
	} else if (strcmp(buff, "btime") == 0) {
	    fscanf(fp, "%d", &stat->btime);
	    //printf("btime = %d\n", stat->btime);
	} else if (strcmp(buff, "intr") == 0) {
	    fscanf(fp, "%d", &stat->intr);
	    //printf("intr = %d\n", stat->intr);
	} else if (strncmp(buff, "cpu", 3) == 0) {
	    stat->cpu_num++;
	}
    } while (fscanf(fp, "%s", buff) == 1);


    fclose(fp);
    stat->last = _proc_now();
    return 1;
}

/**
 * The number of CPUs.
 *
 * @brief The number of CPUs
 * @return On success, returns the number of CPUs.
 * @return On error, -1 is returned.
 * @see proc_cpu_user proc_cpu_nice proc_cpu_system proc_cpu_idle
 */
int proc_cpu_num(void)
{
    errno = 0;
    if ((stat.last == 0) || (_proc_now() > stat.last + _interval)) {
	if (!_proc_read_stat(&stat))
	    return -1;
    }
    return stat.cpu_num;
}

/**
 * The number of pages the system paged in.
 *
 * @brief The number of pages the system paged in
 * @return On success, returns number of pages the system paged in.
 * @return On error, -1 is returned.
 * @see proc_page_out proc_swap_in proc_swap_out proc_intr proc_ctxt
 */
int proc_page_in(void)
{
    errno = 0;
    if ((stat.last == 0) || (_proc_now() > stat.last + _interval)) {
	if (!_proc_read_stat(&stat))
	    return -1;
    }
    return stat.page_in;
}

/**
 * The number of pages the system paged out.
 *
 * @brief The number of pages the system paged out
 * @return On success, returns number of pages the system paged out.
 * @return On error, -1 is returned.
 * @see proc_page_in proc_swap_in proc_swap_out proc_intr proc_ctxt
 */
int proc_page_out(void)
{
    errno = 0;
    if ((stat.last == 0) || (_proc_now() > stat.last + _interval)) {
	if (!_proc_read_stat(&stat))
	    return -1;
    }
    return stat.page_out;
}

/**
 * The number of swap pages that have been brought in.
 *
 * @brief Swap pages that have been brought in
 * @return On success, returns the number of swap pages that have been brought in.
 * @return On error, -1 is returned.
 * @see proc_swap_out proc_page_in proc_page_out proc_intr proc_ctxt
 */
int proc_swap_in(void)
{
    errno = 0;
    if ((stat.last == 0) || (_proc_now() > stat.last + _interval)) {
	if (!_proc_read_stat(&stat))
	    return -1;
    }
    return stat.swap_in;
}

/**
 * The number of swap pages that have been brought out.
 *
 * @brief Swap pages that have been brought out
 * @return On success, returns the number of swap pages that have been brought out.
 * @return On error, -1 is returned.
 * @see proc_swap_in proc_page_in proc_page_out proc_intr proc_ctxt
 */
int proc_swap_out(void)
{
    errno = 0;
    if ((stat.last == 0) || (_proc_now() > stat.last + _interval)) {
	if (!_proc_read_stat(&stat))
	    return -1;
    }
    return stat.swap_out;
}

/**
 * The number of interrupts received, since the system boot.
 *
 * @brief The number of interrupts received
 * @return On success, returns the number of interrupts received.
 * @return On error, -1 is returned.
 * @see proc_ctxt proc_swap_in proc_swap_out proc_page_in proc_page_out
 */
int proc_intr(void)
{
    errno = 0;
    if ((stat.last == 0) || (_proc_now() > stat.last + _interval)) {
	if (!_proc_read_stat(&stat))
	    return -1;
    }
    return stat.intr;
}

/**
 * The number of context switches that the system underwent.
 *
 * @brief The number of context switches
 * @return On success, returns the number of context switches that the system underwent.
 * @return On error, -1 is returned.
 * @see proc_intr proc_swap_in proc_swap_out proc_page_in proc_page_out
 */
int proc_ctxt(void)
{
    errno = 0;
    if ((stat.last == 0) || (_proc_now() > stat.last + _interval)) {
	if (!_proc_read_stat(&stat))
	    return -1;
    }
    return stat.ctxt;
}

/**
 * The boot time since the Epoch (00:00:00 UTC, January 1, 1970), measured in seconds.
 *
 * @brief Boot time
 * @return On success, returns the boot time (in seconds).
 * @return On error, -1 is returned.
 * @see proc_time
 */
int proc_btime(void)
{
    errno = 0;
    if ((stat.last == 0) || (_proc_now() > stat.last + _interval)) {
	if (!_proc_read_stat(&stat))
	    return -1;
    }
    return stat.btime;
}


cpu_t cpu = { 0, 0, 0, 0, 0, 0, 0, 0, 0 };

/* usada apenas para as funcoes proc_jiffies_*() */
cpu_t jiffies = { 0, 0, 0, 0, 0 };

/* le os jiffies acumulados da CPU, em caso de erro retorna 0
 */
int _proc_read_cpu(cpu_t * cpu)
{
    FILE *fp;

    fp = fopen(STAT, "r");
    if (!fp)
	return 0;
    if (fscanf(fp, "%*s %d %d %d %d", &cpu->u, &cpu->n, &cpu->s, &cpu->i)
	!= 4) {
	fclose(fp);
	return 0;
    }
    fclose(fp);
    cpu->last = _proc_now();
    return 1;
}

/**
 * Jiffies spent in user mode.
 *
 * @brief Jiffies spent in user mode
 * @return On success, returns jiffies spent in user mode.
 * @return On error, -1 is returned.
 * @see proc_jiffies_nice proc_jiffies_system proc_jiffies_idle
 * @see proc_cpu_user proc_cpu_nice proc_cpu_system proc_cpu_idle
 */
int proc_jiffies_user(void)
{
    errno = 0;
    if ((jiffies.last == 0) || (_proc_now() > jiffies.last + _interval)) {
	if (!_proc_read_cpu(&jiffies))
	    return -1;
    }
    return jiffies.u;
}

/**
 * Jiffies spent in nice mode (user mode with low priority).
 *
 * @brief Jiffies spent in nice mode
 * @return On success, returns jiffies spent in nice mode.
 * @return On error, -1 is returned.
 * @see proc_jiffies_user proc_jiffies_system proc_jiffies_idle
 * @see proc_cpu_user proc_cpu_nice proc_cpu_system proc_cpu_idle
 */
int proc_jiffies_nice(void)
{
    errno = 0;
    if ((jiffies.last == 0) || (_proc_now() > jiffies.last + _interval)) {
	if (!_proc_read_cpu(&jiffies))
	    return -1;
    }
    return jiffies.n;
}

/**
 * Jiffies spent in system mode.
 *
 * @brief Jiffies spent in system mode
 * @return On success, returns jiffies spent in system mode.
 * @return On error, -1 is returned.
 * @see proc_jiffies_user proc_jiffies_nice proc_jiffies_idle
 * @see proc_cpu_user proc_cpu_nice proc_cpu_system proc_cpu_idle
 */
int proc_jiffies_system(void)
{
    errno = 0;
    if ((jiffies.last == 0) || (_proc_now() > jiffies.last + _interval)) {
	if (!_proc_read_cpu(&cpu))
	    return -1;
    }
    return jiffies.s;
}

/**
 * Jiffies spent in idle process.
 *
 * @brief Jiffies spent in idle process
 * @return On success, returns jiffies spent in idle process.
 * @return On error, -1 is returned.
 * @see proc_jiffies_user proc_jiffies_system proc_jiffies_nice
 * @see proc_cpu_user proc_cpu_nice proc_cpu_system proc_cpu_idle
 */
int proc_jiffies_idle(void)
{
    errno = 0;
    if ((jiffies.last == 0) || (_proc_now() > jiffies.last + _interval)) {
	if (!_proc_read_cpu(&jiffies))
	    return -1;
    }
    return jiffies.i;
}


/* calcula a porcentagem de CPU usando os dados atuais do arquivo /proc/stat
 * e os dados de uma leitura anterior armazenados na struct cpu, essa funcao 
 * preenche uma struct cpu_t contendo as porcentagens de user, nice, system e
 * idle, em caso de erro as funcao retorna 0
 */
int _proc_cpu_status(cpu_t * cpu)
{
    int t1, t2;
    cpu_t cpuj;

    if (!_proc_read_cpu(&cpuj))
	return 0;

    t1 = cpu->u + cpu->n + cpu->s + cpu->i;
    t2 = cpuj.u + cpuj.n + cpuj.s + cpuj.i;

    if (t1 == 0) {
	/* a primeira vez que a funcao for chamada vai dormir
	   por _SLEEP segundos */
	cpu->u = cpuj.u;
	cpu->n = cpuj.n;
	cpu->s = cpuj.s;
	cpu->i = cpuj.i;
	cpu->last = cpuj.last;
	sleep(_SLEEP);
	return _proc_cpu_status(cpu);
    } else {
	if (t2 - t1 != 0) {
	    cpu->user = ((cpuj.u - cpu->u) / (double) (t2 - t1)) * 100;
	    cpu->nice = ((cpuj.n - cpu->n) / (double) (t2 - t1)) * 100;
	    cpu->system = ((cpuj.s - cpu->s) / (double) (t2 - t1)) * 100;
	    cpu->idle = ((cpuj.i - cpu->i) / (double) (t2 - t1)) * 100;
	} else {
	    sleep(_SLEEP);
	    return _proc_cpu_status(cpu);
	}
    }
    cpu->u = cpuj.u;
    cpu->n = cpuj.n;
    cpu->s = cpuj.s;
    cpu->i = cpuj.i;
    cpu->last = _proc_now();
    return 1;
}

/**
 * Percentage of CPU time spent in user mode.
 *
 * @brief Percentage of CPU time spent in user mode
 * @return On success, returns percentage of CPU time spent in user mode.
 * @return On error, -1 is returned.
 * @see proc_cpu_nice proc_cpu_system proc_cpu_idle
 */
double proc_cpu_user(void)
{
    errno = 0;
    if ((cpu.last == 0) || (_proc_now() > cpu.last + _interval)) {
	if (!_proc_cpu_status(&cpu))
	    return -1;
    }
    //_proc_cpu_status(&cpu);

    return cpu.user;
}

/**
 * Percentage of CPU time spent in nice mode (user mode with low priority).
 *
 * @brief Percentage of CPU time spent in nice mode
 * @return On success, returns percentage of CPU time spent in nice mode.
 * @return On error, -1 is returned.
 * @see proc_cpu_user proc_cpu_system proc_cpu_idle
 */
double proc_cpu_nice(void)
{
    errno = 0;
    if ((cpu.last == 0) || (_proc_now() > cpu.last + _interval)) {
	if (!_proc_cpu_status(&cpu))
	    return -1;
    }
    return cpu.nice;
}

/**
 * Percentage of CPU time spent in system mode.
 *
 * @brief Percentage of CPU time spent in system mode
 * @return On success, returns percentage of CPU time spent in system mode.
 * @return On error, -1 is returned.
 * @see proc_cpu_user proc_cpu_nice proc_cpu_idle
 */
double proc_cpu_system(void)
{
    errno = 0;
    if ((cpu.last == 0) || (_proc_now() > cpu.last + _interval)) {
	if (!_proc_cpu_status(&cpu))
	    return -1;
    }
    return cpu.system;
}

/**
 * Percentage of CPU time spent in idle process.
 *
 * @brief Percentage of CPU time spent inidle process
 * @return On success, returns percentage of CPU time spent in idle process.
 * @return On error, -1 is returned.
 * @see proc_cpu_user proc_cpu_nice proc_cpu_system
 */
double proc_cpu_idle(void)
{
    errno = 0;
    if ((cpu.last == 0) || (_proc_now() > cpu.last + _interval)) {
	if (!_proc_cpu_status(&cpu))
	    return -1;
    }
    return cpu.idle;
}

cpuinfo_t cpuinfo = { 0 };

/* le informacoes sobre as CPUs do arquivo /proc/cpuinfo e preenche uma struct
 * cpuinfo contendo o modelo, famila, vendedor, MHz e tamanho de cache,
 * se a maquina tiver mais de uma CPU a funcao preenche o vetor cpu da struct 
 * cpuinfo com n posicoes, onde n  o numero de CPUs dado pela funcao 
 * proc_cpu_num(). 
 * Em caso de erro retorna 0.
 */
int _proc_read_cpuinfo(cpuinfo_t * cpuinfo)
{
    FILE *fp;
    int i = -1;
    char buff[30];
    int n;

    n = proc_cpu_num();
    if (n < 1)
	return 0;
    fp = fopen(CPUINFO, "r");
    if (!fp)
	return 0;

    fscanf(fp, "%s", buff);
    do {
	if (strcmp(buff, "processor") == 0) {
	    i++;
	} else if (strcmp(buff, "vendor_id") == 0) {
	    fscanf(fp, "%*s %s", cpuinfo->cpu[i].vendor_id);
	} else if (strcmp(buff, "cpu") == 0) {
	    fscanf(fp, "%s", buff);
	    if (strcmp(buff, "family") == 0) {
		fscanf(fp, "%*s %d", &cpuinfo->cpu[i].family);
	    } else {
		fscanf(fp, "%*s %f", &cpuinfo->cpu[i].mhz);
	    }
	} else if (strcmp(buff, "bogomips") == 0) {
	    fscanf(fp, "%*s %f", &cpuinfo->cpu[i].bogomips);
	} else if (strcmp(buff, "model") == 0) {
	    fscanf(fp, "%s", buff);
	    if (strcmp(buff, "name") == 0) {
		fscanf(fp, "%s  ", buff);
		fgets(cpuinfo->cpu[i].model_name, TAMMAX, fp);
		cpuinfo->cpu[i].
		    model_name[strlen(cpuinfo->cpu[i].model_name) - 1] =
		    '\0';
	    } else {
		fscanf(fp, "%d", &cpuinfo->cpu[i].model);
	    }
	} else if (strcmp(buff, "cache") == 0) {
	    fscanf(fp, "%*s %*s %d", &cpuinfo->cpu[i].cache_size);
	}
    } while (fscanf(fp, "%s", buff) == 1);

    cpuinfo->last = _proc_now();
    cpuinfo->n_cpu = n;

    fclose(fp);
    return 1;

}

/**
 * Authoritatively identifies the type of processor you have in the system. 
 * For an Intel-based system, place the number in front of "86" to determine
 * the value. This is particularly helpful for those attempting to identify 
 * the architecture of an older system such as a 586, 486, or 386.
 *
 * @brief Family of the processor
 * @param id The processor identifier number. Use 0 on systems that have only one processor.
 * @return On success, returns a identifier of family of the processor.
 * @return On error, -1 is returned.
 * @see proc_cpu_vendor_id proc_cpu_mhz proc_cpu_bogomips 
 * @see proc_cpu_model proc_cpu_model_name proc_cpu_cache_size
 */
int proc_cpu_family(int id)
{
    errno = 0;
    if ((cpuinfo.last == 0) || (_proc_now() > cpuinfo.last + _interval)) {
	if (!_proc_read_cpuinfo(&cpuinfo))
	    return -1;
    }
    if (id < 0 || id > NUMMAX)
	return -1;
    return cpuinfo.cpu[id].family;
}

/**
 * Model of the processor
 *
 * @brief Model of the processor 
 * @param id The processor identifier number. Use 0 on systems that have only one processor.
 * @return On success, returns a identifier of model of the processor.
 * @return On error, -1 is returned.
 * @see proc_cpu_vendor_id proc_cpu_family proc_cpu_mhz proc_cpu_bogomips 
 * @see proc_cpu_model_name proc_cpu_cache_size
 */
int proc_cpu_model(int id)
{
    errno = 0;
    if ((cpuinfo.last == 0) || (_proc_now() > cpuinfo.last + _interval)) {
	if (!_proc_read_cpuinfo(&cpuinfo))
	    return -1;
    }
    if (id < 0 || id > NUMMAX)
	return -1;
    return cpuinfo.cpu[id].model;
}

/**
 * The amount of level 2 memory cache available to the processor.
 *
 * @brief The amount of memory cache
 * @param id The processor identifier number. Use 0 on systems that have only one processor.
 * @return On success, returns the amount of level 2 memory cache available to the processor.
 * @return On error, -1 is returned.
 * @see proc_cpu_vendor_id proc_cpu_family proc_cpu_mhz proc_cpu_bogomips 
 * @see proc_cpu_model proc_cpu_model_name
 */
int proc_cpu_cache_size(int id)
{
    errno = 0;
    if ((cpuinfo.last == 0) || (_proc_now() > cpuinfo.last + _interval)) {
	if (!_proc_read_cpuinfo(&cpuinfo))
	    return -1;
    }
    if (id < 0 || id > NUMMAX)
	return -1;
    return cpuinfo.cpu[id].cache_size;
}

/**
 * The precise speed in megahertz for the processor to the thousandth decimal point.
 *
 * @brief The CPU speed in megahertz
 * @param id The processor identifier number. Use 0 on systems that have only one processor.
 * @return On success, returns the CPU speed in megahertz.
 * @return On error, -1 is returned.
 * @see proc_cpu_vendor_id proc_cpu_family proc_cpu_bogomips proc_cpu_model 
 * @see proc_cpu_model_name proc_cpu_cache_size
 */
float proc_cpu_mhz(int id)
{
    errno = 0;
    if ((cpuinfo.last == 0) || (_proc_now() > cpuinfo.last + _interval)) {
	if (!_proc_read_cpuinfo(&cpuinfo))
	    return -1;
    }
    if (id < 0 || id > NUMMAX)
	return -1;
    return cpuinfo.cpu[id].mhz;
}

/**
 * Bogompis value of the processor. For more information about Bogomips, visit http://www.tldp.org/HOWTO/BogoMips/x78.html#AEN81
 *
 * @brief Bogomips value of the processor
 * @param id The processor identifier number. Use 0 on systems that have only one processor.
 * @return On success, returns the bogomips value of the processor.
 * @return On error, -1 is returned.
 * @see proc_cpu_vendor_id proc_cpu_family proc_cpu_mhz proc_cpu_model
 * @see proc_cpu_model_name proc_cpu_cache_size
 */
float proc_cpu_bogomips(int id)
{
    errno = 0;
    if ((cpuinfo.last == 0) || (_proc_now() > cpuinfo.last + _interval)) {
	if (!_proc_read_cpuinfo(&cpuinfo))
	    return -1;
    }
    if (id < 0 || id > NUMMAX)
	return -1;
    return cpuinfo.cpu[id].bogomips;
}

/**
 * Identifier of vendor of the processor.
 *
 * @brief Identifier of vendor of the processor.
 * @param id The processor identifier number. Use 0 on systems that have only one processor.
 * @return On success, returns a identifier of vendor of the processor.
 * @return On error, -1 is returned.
 * @see proc_cpu_vendor_id proc_cpu_family proc_cpu_bogomips proc_cpu_model 
 * @see proc_cpu_model_name proc_cpu_cache_size
 */
int proc_cpu_vendor_id(int id, char *s)
{
    errno = 0;
    if ((cpuinfo.last == 0) || (_proc_now() > cpuinfo.last + _interval)) {
	if (!_proc_read_cpuinfo(&cpuinfo))
	    return 0;
    }
    if (id < 0 || id > NUMMAX)
	return 0;
    strcpy(s, cpuinfo.cpu[id].vendor_id);
    return 1;
}

/**
 * The common name of the processor, including its project name (eg. AMD Athlon(TM) XP 2200+). 
 *
 * @brief The common name of the processor
 * @param id The processor identifier number. Use 0 on systems that have only one processor.
 * @param s Buffer to put the model name of the processor.
 * @return On success, returns the model name length.
 * @return On error, -1 is returned.
 * @see proc_cpu_vendor_id proc_cpu_family proc_cpu_bogomips proc_cpu_model 
 * @see proc_cpu_cache_size
 */
int proc_cpu_model_name(int id, char *s)
{
    errno = 0;
    if ((cpuinfo.last == 0) || (_proc_now() > cpuinfo.last + _interval)) {
	if (!_proc_read_cpuinfo(&cpuinfo))
	    return 0;
    }
    if (id < 0 || id > NUMMAX)
	return 0;
    strcpy(s, cpuinfo.cpu[id].model_name);
    return 1;
}

net_t net_last = { 0 };
net_t net = { 0 };

int _proc_calculate_net(net_t * net, net_t * old)
{
    int i;
    double diff;
    
    diff = (net->last - old->last) / 1000000;;
    if(diff == 0)
	return -1;
    
    for(i=0;i<net->n_eth;i++) {
	net->eth[i].bytes_in = (net->eth[i].cbytes_in - old->eth[i].cbytes_in) / diff;
	net->eth[i].pkts_in = (net->eth[i].cpkts_in - old->eth[i].cpkts_in) / diff;
	net->eth[i].bytes_out = (net->eth[i].cbytes_out - old->eth[i].cbytes_out) / diff;
	net->eth[i].pkts_out = (net->eth[i].cpkts_out - old->eth[i].cpkts_out) / diff;
	
	old->eth[i].cbytes_in = net->eth[i].cbytes_in;
	old->eth[i].cpkts_in = net->eth[i].cpkts_in;
	old->eth[i].cbytes_out = net->eth[i].cbytes_out;
	old->eth[i].cpkts_out = net->eth[i].cpkts_out;
    }

    old->last = net->last;
    
    return 0;
}

int _proc_read_net(net_t * net, int mode)
{
    FILE *fp;
    char buff[30];
    int eth = 0;

    fp = fopen(NET_DEV, "r");
    if (!fp)
	return 0;

    /* pula as primeiras linhas */
    while ((fscanf(fp, "%s", buff) == 1) && (strncmp(buff, "eth0:",5) != 0));

    /* falta arrumar bystes_in/s e pkts_in/s */

    do {
        if(strlen(buff) > 5) {
		int len, i;
		len = strlen(buff);
		for(i=5;i<len;i++) {
			buff[i-5]=buff[i];
		}
		buff[i-5]='\0';
		sscanf(buff, "%d", &net->eth[eth].cbytes_in);
	} else {
		fscanf(fp, "%d", &net->eth[eth].cbytes_in);	/* bytes */
	}
	fscanf(fp, "%d", &net->eth[eth].cpkts_in);	/* packets */
	fscanf(fp, "%d", &net->eth[eth].errs_in);	/* errs */
	fscanf(fp, "%d", &net->eth[eth].drop_in);	/* drop */
	fscanf(fp, "%d", &net->eth[eth].fifo_in);	/* fifo */
	fscanf(fp, "%d", &net->eth[eth].frame);	/* frame */
	fscanf(fp, "%d", &net->eth[eth].compressed_in);	/* compressed */
	fscanf(fp, "%d", &net->eth[eth].multicast);	/* multicast */

	/* Transmit */
	fscanf(fp, "%d", &net->eth[eth].cbytes_out);	/* bytes */
	fscanf(fp, "%d", &net->eth[eth].cpkts_out);	/* packets */
	fscanf(fp, "%d", &net->eth[eth].errs_out);	/* errs */
	fscanf(fp, "%d", &net->eth[eth].drop_out);	/* drop */
	fscanf(fp, "%d", &net->eth[eth].fifo_out);	/* fifo */
	fscanf(fp, "%d", &net->eth[eth].colls);	/* colls */
	fscanf(fp, "%d", &net->eth[eth].carrier);	/* carrier */
	fscanf(fp, "%d", &net->eth[eth].compressed_out);	/* compressed */

	eth++;
    } while ((fscanf(fp, "%s", buff) == 1)
	     && (strncmp(buff, "eth", 3) == 0));

    net->n_eth = eth;
    net->last = _proc_now();
    fclose(fp); 
    
    if(mode) {
    	if((net->last - net_last.last) < DEFAULT_TIME) {
		usleep(DEFAULT_TIME - (net->last - net_last.last));
		_proc_read_net(net,0);
	}
    	_proc_calculate_net(net, &net_last);
    }
    
    return 1;
}

/**
 * The total number of bytes of data received by the network interface.
 *
 * @brief The total number of bytes of data received by the network interface
 * @param eth network interface number
 * @return On success, returns the total number of bytes of data received by the network interface.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_fifo_in proc_drop_in
 * @see proc_errs_in proc_cpkts_in proc_pkts_in proc_bytes_in
 */
int proc_cbytes_in(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }
  
    return net.eth[eth].cbytes_in;
}

/**
 * The number of bytes per seconds of data received by the network interface.
 *
 * @brief Bytes/s received by the network interface
 * @param eth network interface number.
 * @return On success, returns the number of bytes/s received by the network interface.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_fifo_in
 * @see proc_errs_in proc_cpkts_in proc_pkts_in proc_cbytes_in
 */
double proc_bytes_in(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }
   
    return net.eth[eth].bytes_in;
}

/**
 * The number of packets per seconds received by the network interface.
 *
 * @brief Packets/s received by the network interface
 * @param eth network interface number.
 * @return On success, returns the number of packets/s received by the network interface.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_fifo_in proc_drop_in
 * @see proc_errs_in proc_cpkts_in proc_bytes_in proc_cbytes_in
 */
double proc_pkts_in(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].pkts_in;
}

/**
 * The total number of packets of data received by the network interface.
 *
 * @brief The total number of packets of data received by the network interface
 * @param eth network interface number.
 * @return On success, returns the total number of packets of data received by the network interface.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_fifo_in proc_drop_in
 * @see proc_errs_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_cpkts_in(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].cpkts_in;
}

/**
 * The total number of receive errors detected by the network device driver.
 *
 * @brief The number of receive errors
 * @param eth network interface number.
 * @return On success, returns the total number of receive errors detected by the network device driver.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_fifo_in proc_drop_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_errs_in(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].errs_in;
}

/**
 * The total number of receive packets dropped by the network device driver.
 *
 * @brief The number of receive packets dropped
 * @param eth network interface number.
 * @return On success, returns the total number of receive packets dropped by the network device driver.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_fifo_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_drop_in(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].drop_in;
}

/**
 * The number of receive FIFO buffer errors.
 *
 * @brief The number of receive FIFO buffer errors.
 * @param eth network interface number.
 * @return On success, returns the number of receive FIFO buffer errors.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_fifo_in(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].fifo_in;
}

/**
 * The number of packet framing errors.
 *
 * @brief The number of packet framing errors
 * @param eth network interface number.
 * @return On success, returns the number of packet framing errors.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_fifo_in proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_frame(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].frame;
}

/**
 * The number of compressed packets received by the network device driver.
 *
 * @brief The number of compressed packets received
 * @param eth network interface number.
 * @return On success, returns the number of compressed packets received.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_frame proc_fifo_in proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_compressed_in(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].compressed_in;
}

/**
 * The number of multicast frames transmitted or received by the network device driver.
 *
 * @brief The number of multicast frames transmitted or received
 * @param eth network interface number.
 * @return On success, returns the number of multicast frames transmitted or received.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_compressed_in 
 * @see proc_frame proc_fifo_in proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_multicast(int eth)
{
     errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].multicast;
}

/**
 * The number of bytes per seconds transmitted by the network interface.
 *
 * @brief Bytes/s transmitted by the network interface
 * @param eth network interface number.
 * @return On success, returns the number of bytes/s transmitted by the network interface.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_multicast proc_compressed_in 
 * @see proc_frame proc_fifo_in proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
double proc_bytes_out(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].bytes_out;
}

/**
 * The total number of bytes of data transmitted by the network interface.
 *
 * @brief The total number of bytes of data transmitted by the network interface
 * @param eth network interface number.
 * @return On success, returns the total number of bytes of data transmitted by the network interface.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_bytes_out proc_multicast proc_compressed_in 
 * @see proc_frame proc_fifo_in proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_cbytes_out(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].cbytes_out;
}

/**
 * The number of packets per seconds transmitted by the network interface.
 *
 * @brief Packets/s transmitted by the network interface
 * @param eth network interface number.
 * @return On success, returns the number of packets/s transmitted by the network interface.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_cpkts_in  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_fifo_in proc_drop_in
 * @see proc_errs_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
double proc_pkts_out(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].pkts_out;
}

/**
 * The total number of packets of data transmitted by the network interface.
 *
 * @brief The total number of packets of data transmitted by the network interface
 * @param eth network interface number.
 * @return On success, returns the total number of packets of data transmitted by the network interface.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_pkts_out
 * @see proc_cpkts_in  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_fifo_in proc_drop_in
 * @see proc_errs_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_cpkts_out(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].cpkts_out;
}

/**
 * The total number of transmit errors detected by the network device driver.
 *
 * @brief The number of transmit errors
 * @param eth network interface number.
 * @return On success, returns the total number of transmit errors detected by the network device driver.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_in proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_fifo_in proc_drop_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_errs_out(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].errs_out;
}

/**
 * The total number of transmit packets dropped by the network device driver.
 *
 * @brief The number of transmit packets dropped
 * @param eth network interface number.
 * @return On success, returns the total number of transmit packets dropped by the network device driver.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_in proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_fifo_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_drop_out(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].drop_out;
}

/**
 * The number of transmit FIFO buffer errors.
 *
 * @brief The number of transmit FIFO buffer errors.
 * @param eth network interface number.
 * @return On success, returns the number of transmit FIFO buffer errors.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_in proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_fifo_out(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }
    
    return net.eth[eth].fifo_out;
}

/**
 * The number of collisions detected on the network interface.
 *
 * @brief The number of collisions detected on the network interface
 * @param eth network interface number.
 * @return On success, returns the number of collisions detected on the network interface.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_carrier proc_fifo_out 
 * @see proc_fifo_in proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_colls(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].colls;
}

/**
 * The number of carrier losses detected by the network device driver.
 *
 * @brief The number of carrier losses detected
 * @param eth network interface number.
 * @return On success, returns the number of carrier losses detected by the network device driver.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_compressed_out proc_colls proc_fifo_out 
 * @see proc_fifo_in proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_compressed_in proc_frame proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_carrier(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {;
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].carrier;
}

/**
 * The number of compressed packets transmitted by the network device driver.
 *
 * @brief The number of compressed packets transmitted
 * @param eth network interface number.
 * @return On success, returns the number of compressed packets transmitted.
 * @return On error, -1 is returned.
 * @see proc_eth_num proc_carrier proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_frame proc_fifo_in proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_compressed_out(int eth)
{
    errno = 0;
    
    if (eth < 0 || eth > NUMMAXETH)
	return -1;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.eth[eth].compressed_out;
}

/**
 * The number of network interfaces.
 *
 * @brief The number of network interfaces
 * @return On success, returns the number of network interfaces.
 * @return On error, -1 is returned.
 * @see proc_carrier proc_compressed_out proc_carrier proc_colls 
 * @see proc_fifo_out proc_drop_out proc_errs_out proc_cpkts_out
 * @see proc_pkts_out  proc_cbytes_out proc_bytes_out proc_multicast 
 * @see proc_frame proc_fifo_in proc_drop_in proc_errs_in
 * @see proc_cpkts_in proc_pkts_in proc_bytes_in proc_cbytes_in
 */
int proc_eth_num(void)
{
    errno = 0;
    
    if (net_last.last == 0) {
    	if (!_proc_read_net(&net_last,0))
	    return -1;
    }
    
    if (net.last == 0 || (_proc_now() > net.last + _interval)) {
    	if (!_proc_read_net(&net,1))
	    return -1;
    }

    return net.n_eth;
}

proc_net_con_t tcp = {0,0,0};
proc_net_con_t udp = {0,0,0};

int _proc_net_connections(proc_net_con_t *net_con, char *file)
{
	FILE *fp;
	int num_estab = 0;
	char *line = NULL;
	size_t len = 0;
	int type;

	if(file == NULL || net_con == NULL)
		return -1;
	
	fp = fopen(file, "r");
	if(!fp) {
		return -1;
	}

	net_con->total = 0;
	net_con->established = 0;
	while(getline(&line,&len,fp) != -1) {
		sscanf(line,"%*s %*s %*s %x", &type);
		net_con->total++;
		if(type == 1)
			net_con->established++;
	}
	
	if(line)
		free(line);
	fclose(fp);
	
	net_con->last = _proc_now();
	
	return 0;
}


/**
 * The total number of TCP connections.
 *
 * @brief The total number of TCP connections.
 * @return On success, returns the number of TCP connections.
 * @return On error, -1 is returned.
 * @see proc_tcp_established proc_udp_total proc_udp_established
 */
int proc_tcp_total(void)
{
    errno = 0;
    
    if (tcp.last == 0) {
    	if (_proc_net_connections(&tcp,NET_TCP) < 0)
			return -1;
    }
    
    if (tcp.last == 0 || (_proc_now() > tcp.last + _interval)) {
    	if (_proc_net_connections(&tcp,NET_TCP) < 0)
			return -1;
    }

    return tcp.total;
}

/**
 * The number of TCP connections established.
 *
 * @brief The number of TCP connections established.
 * @return On success, returns the number of TCP connections established.
 * @return On error, -1 is returned.
 * @see proc_tcp_total proc_udp_total proc_udp_established
 */
int proc_tcp_established(void)
{
    errno = 0;
    
    if (tcp.last == 0) {
    	if (_proc_net_connections(&tcp,NET_TCP) < 0)
			return -1;
    }
    
    if (tcp.last == 0 || (_proc_now() > tcp.last + _interval)) {
    	if (_proc_net_connections(&tcp,NET_TCP) < 0)
			return -1;
    }

    return tcp.established;
}

/**
 * The total number of UDP connections.
 *
 * @brief The total number of UDP connections.
 * @return On success, returns the number of UDP connections.
 * @return On error, -1 is returned.
 * @see proc_udp_established  proc_tcp_total proc_tcp_established
 */
int proc_udp_total(void)
{
    errno = 0;
    
    if (udp.last == 0) {
    	if (_proc_net_connections(&udp,NET_UDP) < 0)
			return -1;
    }
    
    if (udp.last == 0 || (_proc_now() > udp.last + _interval)) {
    	if (_proc_net_connections(&udp,NET_UDP) < 0)
			return -1;
    }

    return udp.total;
}

/**
 * The number of UDP connections established.
 *
 * @brief The number of UDP connections established.
 * @return On success, returns the number of UDP connections established.
 * @return On error, -1 is returned.
 * @see proc_udp_total proc_tcp_total proc_tcp_established
 */
int proc_udp_established(void)
{
    errno = 0;
    
    if (udp.last == 0) {
    	if (_proc_net_connections(&udp,NET_UDP) < 0)
			return -1;
    }
    
    if (udp.last == 0 || (_proc_now() > udp.last + _interval)) {
    	if (_proc_net_connections(&udp,NET_UDP) < 0)
			return -1;
    }

    return udp.established;
}

/**
 * The number of users currently logged in to the current host
 *
 * @brief The number of users currently logged
 * @return On success, returns the number of users currently logged.
 * @return On error, -1 is returned.
 */
int proc_users(void)
{
	struct utmp user_slot;
	FILE *fp;
	int num_users = 0;

	fp = fopen(UTMP, "r");
	if(!fp) {
		return -1;
	}
	
	while(fread(&user_slot,sizeof(user_slot),1,fp) == 1) {
		if(user_slot.ut_type==USER_PROCESS)
			num_users++;
	}
	
	fclose(fp);
	
	return num_users;
}

