
/*
 * Biblioteca para ler informacoes do /proc/
 * Funcoes para ler o /proc/PID
 * Marcelo Veiga Neves
 * veiga@inf.ufsm.br
 * Mon Mar 15 03:12:40 2004
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <pwd.h>

#include "proc.h"

#define _SFILE 300
extern double _interval;

/**
 * This function test if a PID exist.
 *
 * @brief Process exists?
 * @param pid PID of process
 * @return If process exist, them 1 is returned, else 0 is returned.
 */
int proc_pid_exists(int pid)
{
    DIR *dd;
    char path[20];
    errno = 0;
    if (pid < 1)
	return 0;
    sprintf(path, "/proc/%d/", pid);
    dd = opendir(path);
    if (!dd)
	return 0;
    closedir(dd);
    return 1;
}

/**
 * The command issued when starting the process.
 *
 * @brief The command issued when starting the process
 * @param pid PID of process
 * @param s buffer to put the command line
 * @return On success, returns the cmd line length. 
 * @return On error, -1 is returned.
 * @see proc_exe_pid proc_root_pid proc_binfmt_pid
 */
int proc_cmdline_pid(int pid, char *s)
{
    char filename[30];
    errno = 0;
    if (pid < 0)
	return 0;
    sprintf(filename, "/proc/%d/cmdline", pid);
    return _proc_read_line(filename, s);
}

/**
 * The binary format of the process (eg, elf).
 *
 * @brief The binary format of the process
 * @param pid PID of process
 * @param s buffer to put the binary format
 * @return On success, returns the binary format length. 
 * @return On error, -1 is returned.
 * @see proc_exe_pid proc_root_pid proc_cmdline_pid
 */
int proc_binfmt_pid(int pid, char *s)
{
    char filename[30];
    errno = 0;
    if (pid < 0)
	return 0;
    sprintf(filename, "/proc/%d/binfmt", pid);
    return _proc_read_line(filename, s);
}

int _proc_read_link(char *path, char *s)
{
    int i;
    errno = 0;
    i = readlink(path, s, _SFILE);
    if (i == -1)
	return 0;
    s[i] = '\0';
    return i;
}

/**
 * The current working directory for the process.
 *
 * @brief The current working directory
 * @param pid PID of process
 * @param s buffer to put the cwd
 * @return On success, returns the cwd length. 
 * @return On error, -1 is returned.
 * @see proc_exe_pid proc_root_pid proc_cmdline_pid
 */
int proc_cwd_pid(int pid, char *s)
{
    char path[_SFILE];
    errno = 0;
    if (pid < 0)
	return 0;
    sprintf(path, "/proc/%d/cwd", pid);
    return _proc_read_link(path, s);
}

/**
 * The executable (with path) of this process.
 *
 * @brief The executable of this process
 * @param pid PID of process
 * @param s buffer to put the executable
 * @return On success, returns the executable length. 
 * @return On error, -1 is returned.
 * @see proc_root_pid proc_cmdline_pid proc_cwd_pid
 */
int proc_exe_pid(int pid, char *s)
{
    char path[_SFILE];
    errno = 0;
    if (pid < 0)
	return 0;
    sprintf(path, "/proc/%d/exe", pid);
    return _proc_read_link(path, s);
}

/**
 * The root directory of the process.
 *
 * @brief The root directory of the process
 * @param pid PID of process
 * @param s buffer to put the root directory
 * @return On success, returns the root dir length. 
 * @return On error, -1 is returned.
 * @see proc_exe_pid proc_cmdline proc_binfmt_pid proc_cwd_pid
 */
int proc_root_pid(int pid, char *s)
{
    char path[_SFILE];
    errno = 0;
    if (pid < 0)
	return 0;
    sprintf(path, "/proc/%d/root", pid);
    return _proc_read_link(path, s);
}

fd_pid_t fd_pid = {
    0, 0
};

int _proc_read_fd_pid(int pid, fd_pid_t * fd_pid)
{
    DIR *dd;
    struct dirent *ed;
    char path[_SFILE];
    int n = -2;			/* para excluir . e .. */
    char buf[_SFILE];
    int i;
    if (pid < 0)
	return 0;
    sprintf(path, "/proc/%d/fd", pid);
    dd = opendir(path);
    if (!dd)
	return 0;
    while ((ed = readdir(dd))) {
	n++;
	if (n > 0) {
	    sprintf(path, "/proc/%d/fd/%s", pid, ed->d_name);
	    i = readlink(path, buf, _SFILE);
	    buf[i] = '\0';
	    strcpy(fd_pid->files[n - 1], buf);
	}
    }
    closedir(dd);
    fd_pid->num = n;
    fd_pid->last = _proc_now();
    return 1;
}

/**
 * The number of allocated file handles by the process.
 *
 * @brief The number of allocated file handles by the process
 * @param pid PID of process
 * @return On success, returns the number of allocated file handles.
 * @return On error, -1 is returned.
 * @see proc_fd_file_pid 
 */
int proc_fd_num_pid(int pid)
{
    errno = 0;
    if ((fd_pid.last == 0) || (_proc_now() > fd_pid.last + _interval)) {
	if (!_proc_read_fd_pid(pid, &fd_pid))
	    return -1;
    }
    return fd_pid.num;
}

/**
 * File name of a allocated file handle.
 *
 * @brief File name of a allocated file handle
 * @param pid PID of process
 * @param id file identifier number (fd)
 * @param buffer to put the file name
 * @return On success, returns the file name lenght.
 * @return On error, -1 is returned.
 * @see proc_fd_num_pid 
 */
int proc_fd_file_pid(int pid, int id, char *s)
{
    errno = 0;
    if ((fd_pid.last == 0) || (_proc_now() > fd_pid.last + _interval)) {
	if (!_proc_read_fd_pid(pid, &fd_pid))
	    return 0;
    }
    if (id < 0 || id > FDMAX)
	return 0;
    strcpy(s, fd_pid.files[id]);
    return strlen(s);
}

statm_pid_t statm_pid = {
    0, -1, -1, -1, -1, -1, -1, -1
};

int _proc_read_statm_pid(int pid, statm_pid_t * statm)
{
    FILE *fp;
    char file[30];
    if (pid < 0)
	return 0;
    sprintf(file, "/proc/%d/statm", pid);
    fp = fopen(file, "r");
    if (!fp)
	return 0;
    fscanf(fp, "%d", &statm->size);
    fscanf(fp, "%d", &statm->resident);
    fscanf(fp, "%d", &statm->shared);
    fscanf(fp, "%d", &statm->trs);
    fscanf(fp, "%d", &statm->drs);
    fscanf(fp, "%d", &statm->lrs);
    fscanf(fp, "%d", &statm->dt);
    statm->last = _proc_now();
    fclose(fp);
    return 1;
}

/**
 * Total program size, in kilobytes.
 *
 * @brief Total program size
 * @param pid PID of process
 * @return On success, returns the total program size, in Kbytes.
 * @return On error, -1 is returned.
 * @see proc_statm_resident_pid proc_statm_shared_pid proc_statm_trs_pid
 * @see proc_statm_drs_pid proc_statm_lrs_pid proc_statm_dt_pid
 */
int proc_statm_size_pid(int pid)
{
    errno = 0;
    if ((statm_pid.last == 0)
	|| (_proc_now() > statm_pid.last + _interval)) {
	if (!_proc_read_statm_pid(pid, &statm_pid))
	    return -1;
    }
    return statm_pid.size;
}

/**
 * Size of memory portions, in kilobytes.
 *
 * @brief Size of memory portions
 * @param pid PID of process
 * @return On success, returns size of memory portions, in Kbytes.
 * @return On error, -1 is returned.
 * @see proc_statm_size_pid proc_statm_shared_pid proc_statm_trs_pid
 * @see proc_statm_drs_pid proc_statm_lrs_pid proc_statm_dt_pid
 */
int proc_statm_resident_pid(int pid)
{
    errno = 0;
    if ((statm_pid.last == 0)
	|| (_proc_now() > statm_pid.last + _interval)) {
	if (!_proc_read_statm_pid(pid, &statm_pid))
	    return -1;
    }
    return statm_pid.resident;
}

/**
 * Number of pages that are shared.
 *
 * @brief Number of pages that are shared
 * @param pid PID of process
 * @return On success, returns the number of pages that are shared.
 * @return On error, -1 is returned.
 * @see proc_statm_size_pid proc_statm_resident_pid proc_statm_trs_pid
 * @see proc_statm_drs_pid proc_statm_lrs_pid proc_statm_dt_pid
 */
int proc_statm_shared_pid(int pid)
{
    errno = 0;
    if ((statm_pid.last == 0)
	|| (_proc_now() > statm_pid.last + _interval)) {
	if (!_proc_read_statm_pid(pid, &statm_pid))
	    return -1;
    }
    return statm_pid.shared;
}

/**
 * Number of pages that are code.
 *
 * @brief Number of pages that are code
 * @param pid PID of process
 * @return On success, returns the number of pages that are code.
 * @return On error, -1 is returned.
 * @see proc_statm_size_pid proc_statm_resident_pid proc_statm_shared_pid
 * @see proc_statm_drs_pid proc_statm_lrs_pid proc_statm_dt_pid
 */
int proc_statm_trs_pid(int pid)
{
    errno = 0;
    if ((statm_pid.last == 0)
	|| (_proc_now() > statm_pid.last + _interval)) {
	if (!_proc_read_statm_pid(pid, &statm_pid))
	    return -1;
    }
    return statm_pid.trs;
}

/**
 * Number of pages of data/stack.
 *
 * @brief Number of pages of data/stack
 * @param pid PID of process
 * @return On success, returns the number of pages of data/stack.
 * @return On error, -1 is returned.
 * @see proc_statm_size_pid proc_statm_resident_pid proc_statm_shared_pid
 * @see proc_statm_trs_pid proc_statm_lrs_pid proc_statm_dt_pid
 */
int proc_statm_drs_pid(int pid)
{
    errno = 0;
    if ((statm_pid.last == 0)
	|| (_proc_now() > statm_pid.last + _interval)) {
	if (!_proc_read_statm_pid(pid, &statm_pid))
	    return -1;
    }
    return statm_pid.drs;
}

/**
 * Number of library pages.
 *
 * @brief Number of library pages
 * @param pid PID of process
 * @return On success, returns the number of library pages.
 * @return On error, -1 is returned.
 * @see proc_statm_size_pid proc_statm_resident_pid proc_statm_shared_pid
 * @see proc_statm_trs_pid proc_statm_drs_pid proc_statm_dt_pid
 */
int proc_statm_lrs_pid(int pid)
{
    errno = 0;
    if ((statm_pid.last == 0)
	|| (_proc_now() > statm_pid.last + _interval)) {
	if (!_proc_read_statm_pid(pid, &statm_pid))
	    return -1;
    }
    return statm_pid.lrs;
}

/**
 * Number of dirty pages.
 *
 * @brief Number of dirty pages
 * @param pid PID of process
 * @return On success, returns the number of dirty pages.
 * @return On error, -1 is returned.
 * @see proc_statm_size_pid proc_statm_resident_pid proc_statm_shared_pid
 * @see proc_statm_trs_pid proc_statm_drs_pid proc_statm_lrs_pid
 */
int proc_statm_dt_pid(int pid)
{
    errno = 0;
    if ((statm_pid.last == 0)
	|| (_proc_now() > statm_pid.last + _interval)) {
	if (!_proc_read_statm_pid(pid, &statm_pid))
	    return -1;
    }
    return statm_pid.dt;
}

stat_pid_t stat_pid = {
    0, -1, "none", -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
};

int _proc_read_stat_pid(int pid, stat_pid_t * s)
{
    FILE *fp;
    char file[30];
    int mem_total;
    if (pid < 0)
	return 0;
    sprintf(file, "/proc/%d/stat", pid);
    fp = fopen(file, "r");
    if (!fp)
	return 0;
    fscanf(fp, "%d %s %c %d", &s->pid, s->exe, &s->state, &s->ppid);
    _proc_clean_name(s->exe);
    fscanf(fp, "%d %d %d %d", &s->pgrp, &s->sid, &s->terminal, &s->ptgid);

    /* faults */
    fscanf(fp, "%lu %lu %lu", &s->flags, &s->min_flt, &s->cmin_flt);
    fscanf(fp, "%lu %lu", &s->maj_flt, &s->cmaj_flt);

    /* cpu jiffies */
    fscanf(fp, "%ld %ld %ld %ld", &s->utime, &s->stime, &s->cutime,
	   &s->cstime);
    fscanf(fp, "%ld %ld", &s->counter, &s->priority);
    fscanf(fp, "%lu %lu %ld", &s->timeout, &s->it_real_value,
	   &s->start_time);
    fscanf(fp, "%u %u %u", &s->vsize, &s->rss, &s->rss_rlim);
    mem_total = proc_mem_total() * 1000;
    s->pused = (s->vsize / (double) mem_total) * 100;

    /* addr */
    fscanf(fp, "%lu %lu", &s->start_code, &s->end_code);
    fscanf(fp, "%lu %lu %lu", &s->start_stack, &s->kstk_esp, &s->kstk_eip);
    fclose(fp);
    return 1;
}

/**
 * The process state: 'R' for running, 'S' for interruible sleeping or 
 * swapping, 'D' for unterruptible sleeping or swapping, 'Z' for zombie, 
 * 'T' for traced or stopped, and 'W' for swapped.
 *
 * @brief The process state
 * @param pid PID of process
 * @return On success, returns the process state.
 * @return On error, -1 is returned.
 * @see proc_exe_name_pid proc_ppid_pid proc_pgrp_pid 
 * @see proc_sid_pid proc_terminal_pid proc_ptgid_pid proc_flags_pid 
 */
char proc_state_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.state;
}

/**
 * The name of the executable file (without path).
 *
 * @brief The name of the executable file
 * @param pid PID of process
 * @param s buffer to put the executable file
 * @return On success, returns the executable file length.
 * @return On error, -1 is returned.
 * @see proc_state_pid proc_ppid_pid proc_pgrp_pid 
 * @see proc_sid_pid proc_terminal_pid proc_ptgid_pid proc_flags_pid 
 */
int proc_exe_name_pid(int pid, char *s)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return 0;
    }
    strcpy(s, stat_pid.exe);
    return strlen(s);
}

/**
 * The PID of the parent process.
 *
 * @brief The PID of the parent process
 * @param pid PID of process
 * @return On success, returns the PID of the parent process.
 * @return On error, -1 is returned.
 * @see proc_state_pid proc_exe_name_pid proc_pgrp_pid 
 * @see proc_sid_pid proc_terminal_pid proc_ptgid_pid proc_flags_pid 
 */
int proc_ppid_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.ppid;
}

/**
 * The process group
 *
 * @brief The process group
 * @param pid PID of process
 * @return On success, returns the process group.
 * @return On error, -1 is returned.
 * @see proc_state_pid proc_exe_name_pid proc_ppid_pid 
 * @see proc_sid_pid proc_terminal_pid proc_ptgid_pid proc_flags_pid 
 */
int proc_pgrp_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.pgrp;
}

/**
 * The SID of the process (session).
 *
 * @brief The SID of the process
 * @param pid PID of process
 * @return On success, returns the SID of the process.
 * @return On error, -1 is returned.
 * @see proc_state_pid proc_exe_name_pid proc_ppid_pid proc_pgrp_pid 
 * @see proc_terminal_pid proc_ptgid_pid proc_flags_pid 
 */
int proc_sid_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.sid;
}

/**
 * The terminal used by the process
 *
 * @brief The terminal used by the process
 * @param pid PID of process
 * @return On success, returns the terminal used by the process.
 * @return On error, -1 is returned.
 * @see proc_state_pid proc_exe_name_pid proc_ppid_pid proc_pgrp_pid 
 * @see proc_sid_pid proc_ptgid_pid proc_flags_pid 
 */
int proc_terminal_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.terminal;
}

/**
 * The process group owned by the terminal used by the process.
 *
 * @param pid PID of process
 * @return On success, returns the process group owned by the terminal.
 * @return On error, -1 is returned.
 * @see proc_state_pid proc_exe_name_pid proc_ppid_pid proc_pgrp_pid 
 * @see proc_sid_pid proc_terminal_pid proc_flags_pid 
 */
int proc_ptgid_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.ptgid;
}

/**
 * The flags of the process.
 *
 * @brief The flags of the process
 * @param pid PID of process
 * @return On success, returns the flags of the process.
 * @return On error, -1 is returned.
 * @see proc_state_pid proc_exe_name_pid proc_ppid_pid proc_pgrp_pid 
 * @see proc_sid_pid proc_terminal_pid proc_ptgid_pid 
 */
unsigned long proc_flags_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.flags;
}

/**
 * The number of minor faults incurred by the process. A minos
 * fault is an error in accessing memory pages which is handled without 
 * accessing external media.
 *
 * @brief The number of minor faults incurred by the process
 * @param pid PID of process
 * @return On success, returns the number of minor faults.
 * @return On error, -1 is returned.
 * @see proc_cmin_flt_pid proc_maj_flt_pid proc_cmaj_flt_pid
 */
unsigned long proc_min_flt_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.min_flt;
}

/**
 * The number of minor faults incurred by the process and its children. A minos
 * fault is an error in accessing memory pages which is handled without 
 * accessing external media.
 *
 * @brief The number of accumulated minor faults
 * @param pid PID of process
 * @return On success, returns the number of accumulated minor faults.
 * @return On error, -1 is returned.
 * @see proc_min_flt_pid proc_maj_flt_pid proc_cmaj_flt_pid
 */
unsigned long proc_cmin_flt_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.cmin_flt;
}

/**
 * The number of major faults incurred by the process. A major
 * fault is an error in accessing memory pages which is handled with 
 * accessing external media.
 *
 * @brief The number of major faults
 * @param pid PID of process
 * @return On success, returns the number of major faults.
 * @return On error, -1 is returned.
 * @see proc_min_flt_pid proc_cmin_flt_pid proc_cmaj_flt_pid
 */
unsigned long proc_maj_flt_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.maj_flt;
}

/**
 * The number of major faults incurred by the process and its children. A major
 * fault is an error in accessing memory pages which is handled with 
 * accessing external media.
 *
 * @brief The number of accumulated major faults
 * @param pid PID of process
 * @return On success, returns the number of accumulated major faults.
 * @return On error, -1 is returned.
 * @see proc_min_flt_pid proc_cmin_flt_pid proc_maj_flt_pid
 */
unsigned long proc_cmaj_flt_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.cmaj_flt;
}

/**
 * The number of jiffies the process spent in user mode.
 *
 * @brief Jiffies spent in user mode by the process
 * @param pid PID of process
 * @return On success, returns the jiffies spent in user mode by the process.
 * @return On error, -1 is returned.
 * @see proc_stime_pid proc_cutime_pid proc_cstime_pid 
 * @see proc_counter_pid proc_priority_pid proc_timeout_pid 
 * @see proc_it_real_value_pid proc_start_time_pid
 */
long int proc_utime_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.utime;
}

/**
 * The number of jiffies the process spent in kernel mode.
 *
 * @brief Jiffies spent in kernel mode by the process
 * @param pid PID of process
 * @return On success, returns the jiffies spent in kernel mode by the process.
 * @return On error, -1 is returned.
 * @see proc_utime_pid proc_cutime_pid proc_cstime_pid 
 * @see proc_counter_pid proc_priority_pid proc_timeout_pid 
 * @see proc_it_real_value_pid proc_start_time_pid
 */
long int proc_stime_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.stime;
}

/**
 * The number of jiffies the process and its children spent in user mode.
 *
 * @brief Jiffies spent in user mode by the process
 * @param pid PID of process
 * @return On success, returns the jiffies spent in kernel mode.
 * @return On error, -1 is returned.
 * @see proc_utime_pid proc_stime_pid proc_cstime_pid 
 * @see proc_counter_pid proc_priority_pid proc_timeout_pid 
 * @see proc_it_real_value_pid proc_start_time_pid
 */
long int proc_cutime_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.cutime;
}

/**
 * The number of jiffies the process and its children spent in kernel mode.
 *
 * @brief Jiffies spent in kernel mode by the process
 * @param pid PID of process
 * @return On success, returns the jiffies spent in kernel mode.
 * @return On error, -1 is returned.
 * @see proc_utime_pid proc_cutime_pid proc_stime_pid 
 * @see proc_counter_pid proc_priority_pid proc_timeout_pid 
 * @see proc_it_real_value_pid proc_start_time_pid
 */
long int proc_cstime_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.cstime;
}

/**
 * The maximum number of jiffies the process can run in a time slice.
 *
 * @brief The maximum number of jiffies in a time slice
 * @param pid PID of process
 * @return On success, returns the maximum number of jiffies in a time slice.
 * @return On error, -1 is returned.
 * @see proc_utime_pid proc_cutime_pid proc_stime_pid 
 * @see proc_cstime_pid proc_priority_pid proc_timeout_pid 
 * @see proc_it_real_value_pid proc_start_time_pid
 */
long int proc_counter_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.counter;
}

/**
 * The UNIX nice value used to calculate a new value for counter (see proc_counter_pid()).
 *
 * @brief The UNIX nice value
 * @param pid PID of process
 * @return On success, returns the UNIX nice value.
 * @return On error, -1 is returned.
 * @see proc_utime_pid proc_cutime_pid proc_stime_pid 
 * @see proc_cstime_pid proc_counter_pid proc_timeout_pid 
 * @see proc_it_real_value_pid proc_start_time_pid
 */
long int proc_priority_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.priority;
}

/**
 * The value in jiffies before triggering a timeout.
 *
 * @brief Jiffies before triggering a timeout
 * @param pid PID of process
 * @return On success, returns the jiffies before triggering a timeout.
 * @return On error, -1 is returned.
 * @see proc_utime_pid proc_cutime_pid proc_stime_pid 
 * @see proc_cstime_pid proc_counter_pid proc_priority_pid 
 * @see proc_it_real_value_pid proc_start_time_pid
 */
unsigned long proc_timeout_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.timeout;
}

/**
 * The value of the interval timer.
 *
 * @brief The value of the interval timer
 * @param pid PID of process
 * @return On success, returns the value of the interval timer.
 * @return On error, -1 is returned.
 * @see proc_utime_pid proc_cutime_pid proc_stime_pid 
 * @see proc_cstime_pid proc_counter_pid proc_priority_pid 
 * @see proc_timeout_pid proc_start_time_pid
 */
unsigned long proc_it_real_value_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.it_real_value;
}

/**
 * The start time of the process in jiffies since system start.
 *
 * @brief The start time of the process
 * @param pid PID of process
 * @return On success, returns the start time of the process.
 * @return On error, -1 is returned.
 * @see proc_utime_pid proc_cutime_pid proc_stime_pid 
 * @see proc_cstime_pid proc_counter_pid proc_priority_pid 
 * @see proc_timeout_pid proc_it_real_value_pid
 */
long int proc_start_time_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.start_time;
}

/**
 * The size in bytes of the memory accessible to the process.
 *
 * @brief The size of the memory accessible to the process
 * @param pid PID of process
 * @return On success, returns the size of the memory accessible to the process.
 * @return On error, -1 is returned.
 * @see proc_rss_rlim_pid proc_rss_pid proc_mem_pused_pid
 * @see proc_start_time_pid
 */
unsigned proc_vsize_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.vsize;
}

/**
 * Percentage of main memory accessible to the process.
 *
 * @brief Percentage of main memory accessible to the process
 * @param pid PID of process
 * @return On success, returns the percentage of main memory accessible to the process.
 * @return On error, -1 is returned.
 * @see proc_rss_rlim_pid proc_rss_pid proc_vsize_pid
 */
double proc_mem_pused_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.pused;
}

/**
 * The number of pages of the process currently in physical memory.
 *
 * @brief The number of pages currently in physical memory
 * @param pid PID of process
 * @return On success, returns the number of pages currently in physical memory.
 * @return On error, -1 is returned.
 * @see proc_rss_rlim_pid proc_mem_pused_pid proc_vsize_pid
 */
unsigned proc_rss_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.rss;
}

/**
 * The maximum number (rlim) of memory pages for the process which may be 
 * simultaneously present in memory.
 *
 * @brief The current rlim value
 * @param pid PID of process
 * @return On success, returns the current //rlim// value.
 * @return On error, -1 is returned.
 * @see proc_rss_pid proc_mem_pused_pid proc_vsize_pid
 */
unsigned proc_rss_rlim_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.rss_rlim;
}

/**
 * The start address of the text segment.
 *
 * @brief The start address of the text segment
 * @param pid PID of process
 * @return On success, returns the start address of the text segment.
 * @return On error, -1 is returned.
 * @see proc_kstk_eip_pid proc_kstk_esp_pid proc_start_stack_pid 
 * @see proc_end_code_pid
 */
unsigned long proc_start_code_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.start_code;
}

/**
 * The end address of the text segment.
 *
 * @brief The end address of the text segment
 * @param pid PID of process
 * @return On success, returns the end address of the text segment.
 * @return On error, -1 is returned.
 * @see proc_kstk_eip_pid proc_kstk_esp_pid proc_start_stack_pid 
 */
unsigned long proc_end_code_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.end_code;
}

/**
 * The start address of the stack.
 *
 * @brief The start address of the stack
 * @param pid PID of process
 * @return On success, returns the start address of the stack.
 * @return On error, -1 is returned.
 * @see proc_kstk_eip_pid proc_kstk_esp_pid proc_end_code_pid 
 */
unsigned long proc_start_stack_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.start_stack;
}

/**
 * The current stack pointer (ESP) of the process.
 *
 * @brief The current stack pointer of the process
 * @param pid PID of process
 * @return On success, returns the current stack pointer of the process.
 * @return On error, -1 is returned.
 * @see proc_kstk_eip_pid proc_end_code_pid proc_start_stack_pid 
 */
unsigned long proc_kstk_esp_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.kstk_esp;
}

/**
 * The current instruction pointer (EIP) of the process.
 *
 * @brief The current stack pointer of the process
 * @param pid PID of process
 * @return On success, returns the current instruction pointer of the process.
 * @return On error, -1 is returned.
 * @see proc_kstk_esp_pid proc_end_code_pid proc_start_stack_pid 
 */
unsigned long proc_kstk_eip_pid(int pid)
{
    errno = 0;
    if ((stat_pid.last == 0)
	|| (_proc_now() > stat_pid.last + _interval)) {
	if (!_proc_read_stat_pid(pid, &stat_pid))
	    return -1;
    }
    return stat_pid.kstk_eip;
}

cpu_t jif = {
    0, 0, 0, 0, 0
};

cpu_pid_t cpu_pid = {
    0, 0, 0, 0, 0, 0, 0, 0, 0
};

int _proc_cpu_pid(int pid, cpu_pid_t * cpu_pid, cpu_t * cpu)
{
    int t1, t2;
    cpu_t cpuj;
    long int utime, stime;
    if (!_proc_read_cpu(&cpuj))
	return 0;
    t1 = cpu->u + cpu->n + cpu->s + cpu->i;
    t2 = cpuj.u + cpuj.n + cpuj.s + cpuj.i;
    utime = proc_utime_pid(pid);
    stime = proc_stime_pid(pid);
    if (t1 == 0) {

	/* a primeira vez que a funcao for chamada vai dormir
	   por _SLEEP segundos */
	cpu->u = cpuj.u;
	cpu->n = cpuj.n;
	cpu->s = cpuj.s;
	cpu->i = cpuj.i;
	cpu->last = cpuj.last;
	cpu_pid->utime = utime;
	cpu_pid->stime = stime;
	cpu_pid->last = _proc_now();
	sleep(_SLEEP);
	return _proc_cpu_pid(pid, cpu_pid, cpu);
    } else {
	if (t2 - t1 != 0) {
	    cpu_pid->puser =
		((utime - cpu_pid->utime) / (double) (t2 - t1)) * 100;
	    cpu_pid->psystem =
		((stime - cpu_pid->stime) / (double) (t2 - t1)) * 100;
	} else {
	    sleep(_SLEEP);
	    return _proc_cpu_pid(pid, cpu_pid, cpu);
	}
    }
    cpu->u = cpuj.u;
    cpu->n = cpuj.n;
    cpu->s = cpuj.s;
    cpu->i = cpuj.i;
    cpu->last = _proc_now();
    cpu_pid->utime = utime;
    cpu_pid->stime = stime;
    cpu_pid->last = _proc_now();
    return 1;
}

/**
 * Percentage of CPU time spent in user mode by the process.
 *
 * @brief Percentage of CPU time spent in user mode by the process
 * @return On success, returns percentage of CPU time spent in user mode.
 * @return On error, -1 is returned.
 * @see proc_cpu_system_pid
 */
double proc_cpu_user_pid(int pid)
{
    errno = 0;
    if ((cpu_pid.last == 0) || (_proc_now() > cpu_pid.last + _interval)) {
	if (!_proc_cpu_pid(pid, &cpu_pid, &jif))
	    return -1;
    }
    return cpu_pid.puser;
}

/**
 * Percentage of CPU time spent in system (kernel) mode by the process.
 *
 * @brief Percentage of CPU time spent in system mode by the process
 * @return On success, returns percentage of CPU time spent in system mode.
 * @return On error, -1 is returned.
 * @see proc_cpu_user_pid
 */
double proc_cpu_system_pid(int pid)
{
    errno = 0;
    if ((cpu_pid.last == 0) || (_proc_now() > cpu_pid.last + _interval)) {
	if (!_proc_cpu_pid(pid, &cpu_pid, &jif))
	    return -1;
    }
    return cpu_pid.psystem;
}

/**
 * The user ID of the owner of the process
 *
 * @brief The user ID of the owner of the process
 * @param pid PID of process
 * @return On success, returns the user ID of the owner.
 * @return On error, -1 is returned.
 * @see proc_username_pid
 */
int proc_uid_pid(int pid)
{
    char filename[30];
    char buff[30];
    FILE *fp;
    errno = 0;
    if (pid < 0)
	return 0;
    sprintf(filename, "/proc/%d/status", pid);
    fp = fopen(filename, "r");
    if (!fp)
	return -1;
    while (fscanf(fp, "%s", buff) == 1 && !feof(fp)) {
	if (strcmp("Uid:", buff) == 0) {
	    fscanf(fp, "%s", buff);
	    fclose(fp);
	    return atoi(buff);
	}
    }
    fclose(fp);
    return -1;
}

/**
 * The user name of the owner of the process
 *
 * @brief The user name of the owner of the process
 * @param pid PID of process
 * @param s buffer to put the user name
 * @return On success, returns the user name length.
 * @return On error, -1 is returned.
 * @see proc_uid_pid
 */
int proc_username_pid(int pid, char *s)
{
	struct passwd *pw;
        int uid;
	
	errno = 0;
	if (pid < 0)
		return 0;
	
	if(s == NULL) 
		return 0;
	
	uid = proc_uid_pid(pid);	
	pw = getpwuid(uid);
	strcpy(s,pw->pw_name);
	
	return 1;
}
